-------------------------------------------------------------------
-- KSI Labs, LLC CENTIPEDE-PCI Mach 3 specific stuff.            --
--                                                               --
-- This is Mach 3 Index (position) registers.                    --
--                                                               --
-- Positions are INCremented when axis DIR is 0 and REV bit is   --
-- NOT set ('0') for the corresponding axis in mach3_ctl i.e.    --
-- when those bits are NOT equal and decremented otherwise on    --
-- every pulse output. That is how it works in Mach3.            --
--                                                               --
-- There is a transparent latch between the actual Index         --
-- register and external read bus. It is transparent during      --
-- operation but it latches the current state on Home switch hit --
-- or Probe input (Digitize). Home switch is per-axis and only   --
-- corresponding axis is latched, Digitize switch is a single    --
-- one and ALL axes are latched when it's been hit. Latched      --
-- state is cancelled by Index register read from PCI (they are  --
-- independent i.e. that is per-register cancellation.)          --
--                                                               --
-- All READs are served by the transparent latches, all WRITEs   --
-- go to the actual Index registers behind the latches.          --
-------------------------------------------------------------------

--------------------------------------------------------------------------
--			LICENSE and Terms of Use.			--
--									--
-- First of all, this is NOT GPL or Free Software. It is open so you	--
-- have the VHDL source code. There is also the source code for the	--
-- Windows Engine for Mach3 CNC software available from KSI Labs, LLC	--
-- website, http://www.ksilabs.com.					--
--									--
-- You are free to use this particular VHDL code, make modifications to	--
-- customize it for your own needs, fix inevitable bugs, port it to 	--
-- other CNC software and do anything else as long as your derivative	--
-- work that is based on this code or uses parts or pieces thereof is	--
-- made to be used with any KSI Labs, LLC products.			--
--									--
-- We do NOT grant you a permission to use any parts of this code in	--
-- any products that are designed for products not manufactured or	--
-- distributed by KSI Labs, LLC. We might give such a permission on	--
-- case-by-case basis but it is not legal to use this code or bits and	--
-- pieces thereof with products not coming from KSI Labs, LLC without	--
-- our explicit written permission.					--
--									--
-- The original copyright notices must be included in all derivative	--
-- works. You do NOT have to ask our permission to make any changes to	--
-- this code and you are not under obligation to give us any of your	--
-- work back as long as you are using it for your own needs.		--
--									--
-- However we do NOT give our permission to distribute your derivative	--
-- work unless all original copyrights are retained, all the conditions	--
-- of this license are met (use only with KSI Labs, LLC products) and	--
-- full source code for such derivative work is available for free to	--
-- everybody without any further conditions including but not limited	--
-- to signing any kinds of NDA. We explicitely deny any permission to	--
-- distribute any derivative work based on this code with closed source	--
-- binary-only fragments or include bits or pieces of this code in such	--
-- products. That essentially means that it is NOT legal to use it with --
-- ANY proprietary HDL megafunctions, megacores, etc. if the resulting	--
-- product is distributed. It IS legal to use whatever you want if the	--
-- resulting product is made for use with a KSI Labs, LLC product AND	--
-- that resulting product is not distributed. Exclusions can be granted --
-- by KSI Labs, LLC on case-by-case basis.				--
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.mach3_data_types.all;


entity mach3_index is
port (
	data_in		: in std_logic_vector(31 downto 0);
	steps		: in std_logic_vector(5 downto 0);
	step_stb	: in std_logic;
	dirs		: in std_logic_vector(5 downto 0);
	idx_regs	: out index_regs;
	home_sw_hit	: in std_logic_vector(5 downto 0);
	probe_hit	: in std_logic;
	idx_wr		: in std_logic_vector(5 downto 0);
	clk		: in std_logic;
	rst		: in std_logic
);
end mach3_index;

architecture guts of mach3_index is

	-------------
	-- Signals --
	-------------

	type idx_int_regs is array (0 to 5) of integer;
	signal idx	: idx_int_regs;

begin

	cntr_proc : process (rst, clk)
	begin
		if (rst='1') then
			for i in 0 to 5 loop
				idx(i) <= 0;
			end loop;
		elsif (rising_edge(clk)) then
			for i in 0 to 5 loop
				if (idx_wr(i)='1') then
					idx(i) <= to_integer(signed(data_in));
				elsif ((steps(i) and step_stb)='1') then
					if (dirs(i)='1') then
						idx(i) <= idx(i) - 1;
					else
						idx(i) <= idx(i) + 1;
					end if;
				end if;
			end loop;
		end if;
	end process;


	latch_proc : process (rst, clk)
	begin
		if (rst='1') then
			for i in 0 to 5 loop
				idx_regs(i) <= x"00000000";
			end loop;
		elsif (rising_edge(clk)) then
			for i in 0 to 5 loop
				if (probe_hit='0' and home_sw_hit(i)='0') then
					idx_regs(i) <= std_logic_vector(to_signed(idx(i),32));
				end if;
			end loop;
		end if;
	end process;

end guts;
