-------------------------------------------------------------------
-- KSI Labs, LLC CENTIPEDE-PCI Mach 3 specific stuff.            --
--                                                               --
-- This is Mach 3 specific stuff. Limit/Home/Probe switches.     --
-------------------------------------------------------------------

--------------------------------------------------------------------------
--			LICENSE and Terms of Use.			--
--									--
-- First of all, this is NOT GPL or Free Software. It is open so you	--
-- have the VHDL source code. There is also the source code for the	--
-- Windows Engine for Mach3 CNC software available from KSI Labs, LLC	--
-- website, http://www.ksilabs.com.					--
--									--
-- You are free to use this particular VHDL code, make modifications to	--
-- customize it for your own needs, fix inevitable bugs, port it to 	--
-- other CNC software and do anything else as long as your derivative	--
-- work that is based on this code or uses parts or pieces thereof is	--
-- made to be used with any KSI Labs, LLC products.			--
--									--
-- We do NOT grant you a permission to use any parts of this code in	--
-- any products that are designed for products not manufactured or	--
-- distributed by KSI Labs, LLC. We might give such a permission on	--
-- case-by-case basis but it is not legal to use this code or bits and	--
-- pieces thereof with products not coming from KSI Labs, LLC without	--
-- our explicit written permission.					--
--									--
-- The original copyright notices must be included in all derivative	--
-- works. You do NOT have to ask our permission to make any changes to	--
-- this code and you are not under obligation to give us any of your	--
-- work back as long as you are using it for your own needs.		--
--									--
-- However we do NOT give our permission to distribute your derivative	--
-- work unless all original copyrights are retained, all the conditions	--
-- of this license are met (use only with KSI Labs, LLC products) and	--
-- full source code for such derivative work is available for free to	--
-- everybody without any further conditions including but not limited	--
-- to signing any kinds of NDA. We explicitely deny any permission to	--
-- distribute any derivative work based on this code with closed source	--
-- binary-only fragments or include bits or pieces of this code in such	--
-- products. That essentially means that it is NOT legal to use it with --
-- ANY proprietary HDL megafunctions, megacores, etc. if the resulting	--
-- product is distributed. It IS legal to use whatever you want if the	--
-- resulting product is made for use with a KSI Labs, LLC product AND	--
-- that resulting product is not distributed. Exclusions can be granted --
-- by KSI Labs, LLC on case-by-case basis.				--
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.mach3_data_types.all;


entity mach3_switches is
port (
	sw_probe	: in std_logic;
	sw_home		: in std_logic_vector(5 downto 0);
	sw_lim_neg	: in std_logic_vector(5 downto 0);
	sw_lim_pos	: in std_logic_vector(5 downto 0);
	lim_neg_hit	: out std_logic_vector(5 downto 0);
	lim_pos_hit	: out std_logic_vector(5 downto 0);
	probe_hit	: out std_logic;
	home_hit	: out std_logic_vector(5 downto 0);
	lim_neg_clr	: in std_logic_vector(5 downto 0);
	lim_pos_clr	: in std_logic_vector(5 downto 0);
	probe_clr	: in std_logic;
	home_clr	: in std_logic_vector(5 downto 0);
	rst		: in std_logic;
	clk		: in std_logic
);
end mach3_switches;

architecture guts of mach3_switches is

	-------------
	-- Signals --
	-------------

begin

	limit_switches_proc : process (rst, clk)
	begin
		if (rst='1') then
			for i in 0 to 5 loop
				lim_pos_hit(i) <= '0';
				lim_neg_hit(i) <= '0';
				home_hit(i) <= '0';
			end loop;
			probe_hit <= '0';
		elsif (rising_edge(clk)) then
			for i in 0 to 5 loop
				if (sw_lim_pos(i)='1') then
					lim_pos_hit(i) <= '1';
				elsif (lim_pos_clr(i)='1') then
					lim_pos_hit(i) <= '0';
				end if;
				if (sw_lim_neg(i)='1') then
					lim_neg_hit(i) <= '1';
				elsif (lim_neg_clr(i)='1') then
					lim_neg_hit(i) <= '0';
				end if;
				if (sw_home(i)='1') then
					home_hit(i) <= '1';
				elsif (home_clr(i)='1') then
					home_hit(i) <= '0';
				end if;
			end loop;
			if (sw_probe='1') then
				probe_hit <= '1';
			elsif (probe_clr='1') then
				probe_hit <= '0';
			end if;
		end if;
	end process;

end guts;
