-------------------------------------------------------------------
-- KSI Labs, LLC CENTIPEDE-PCI Mach 3 specific stuff.            --
--                                                               --
-- This is Mach 3 specific stuff. Pulsing engine, positions etc. --
-- ALL components and Mach3 specific data types are here.        --
-------------------------------------------------------------------

--------------------------------------------------------------------------
--			LICENSE and Terms of Use.			--
--									--
-- First of all, this is NOT GPL or Free Software. It is open so you	--
-- have the VHDL source code. There is also the source code for the	--
-- Windows Engine for Mach3 CNC software available from KSI Labs, LLC	--
-- website, http://www.ksilabs.com.					--
--									--
-- You are free to use this particular VHDL code, make modifications to	--
-- customize it for your own needs, fix inevitable bugs, port it to 	--
-- other CNC software and do anything else as long as your derivative	--
-- work that is based on this code or uses parts or pieces thereof is	--
-- made to be used with any KSI Labs, LLC products.			--
--									--
-- We do NOT grant you a permission to use any parts of this code in	--
-- any products that are designed for products not manufactured or	--
-- distributed by KSI Labs, LLC. We might give such a permission on	--
-- case-by-case basis but it is not legal to use this code or bits and	--
-- pieces thereof with products not coming from KSI Labs, LLC without	--
-- our explicit written permission.					--
--									--
-- The original copyright notices must be included in all derivative	--
-- works. You do NOT have to ask our permission to make any changes to	--
-- this code and you are not under obligation to give us any of your	--
-- work back as long as you are using it for your own needs.		--
--									--
-- However we do NOT give our permission to distribute your derivative	--
-- work unless all original copyrights are retained, all the conditions	--
-- of this license are met (use only with KSI Labs, LLC products) and	--
-- full source code for such derivative work is available for free to	--
-- everybody without any further conditions including but not limited	--
-- to signing any kinds of NDA. We explicitely deny any permission to	--
-- distribute any derivative work based on this code with closed source	--
-- binary-only fragments or include bits or pieces of this code in such	--
-- products. That essentially means that it is NOT legal to use it with --
-- ANY proprietary HDL megafunctions, megacores, etc. if the resulting	--
-- product is distributed. It IS legal to use whatever you want if the	--
-- resulting product is made for use with a KSI Labs, LLC product AND	--
-- that resulting product is not distributed. Exclusions can be granted --
-- by KSI Labs, LLC on case-by-case basis.				--
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

package mach3_data_types is
	type index_adr is array(0 to 5) of std_logic_vector(6 downto 0);
	type index_regs is array(0 to 5) of std_logic_vector(31 downto 0);
end mach3_data_types;

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.mach3_data_types.all;

package mach3_components is
	component pci32ttiny
	generic (
		vendorID	: std_logic_vector(15 downto 0)	:= x"feff";
		deviceID	: std_logic_vector(15 downto 0)	:= x"0001";
		revisionID	: std_logic_vector(7 downto 0)	:= x"00";
		subsystemID	: std_logic_vector(15 downto 0)	:= x"0202";
		subsystemvID	: std_logic_vector(15 downto 0)	:= x"1172";
		classcodeID	: std_logic_vector(23 downto 0)	:= x"078000"
	);
	port (
		-- General 
		pci_clk		: in std_logic;
		pci_rst		: in std_logic;
    
		-- PCI target 32bits
		pci_ad		: inout std_logic_vector(31 downto 0);
		pci_cbe		: in std_logic_vector(3 downto 0);
		pci_par		: out std_logic;  
		pci_frame	: in std_logic;
		pci_irdy	: in std_logic;
		pci_trdy	: out std_logic;
		pci_devsel	: out std_logic;
		pci_stop	: out std_logic;
		pci_idsel	: in std_logic;
		pci_perr	: out std_logic;
		pci_serr	: out std_logic;
		pci_inta	: out std_logic;
      
		-- Local bus
		lb_adr		: out std_logic_vector(11 downto 0);     
		lb_data_in	: out std_logic_vector(31 downto 0);
		lb_data_out	: in std_logic_vector(31 downto 0);
		lb_nbe		: out std_logic_vector(3 downto 0);
		lb_nint		: in std_logic;
		lb_rd		: out std_logic;
		lb_wr		: out std_logic
	);
	end component;

	component tlv5617a_dac
	generic (
		clkdiv_val	: unsigned(5 downto 0) := b"10_0010"
	);
	port (
		dac_idle	: out std_logic;
		ss_pin		: out std_logic;
		sclk_pin	: out std_logic;
		mosi_pin	: out std_logic;
		dac_thr		: in std_logic_vector(15 downto 0);
		dac_sel		: in std_logic;
		wr		: in std_logic;
		clk		: in std_logic;
		rst		: in std_logic
	);
	end component;

	component tlv1544_adc
	generic (
		clkdiv_val	: unsigned(8 downto 0) := b"0_0010_0010";
		convtime_val	: unsigned(8 downto 0) := b"1_1111_0100"
	);
	port (
		adc_idle	: out std_logic;
		adc_rhr		: out std_logic_vector(9 downto 0);
		adc_thr		: in std_logic_vector(9 downto 0);
		ss_pin		: out std_logic;
		sclk_pin	: out std_logic;
		mosi_pin	: out std_logic;
		miso_pin	: in std_logic;
		adc_sel		: in std_logic;
		wr		: in std_logic;
		clk		: in std_logic;
		rst		: in std_logic
	);
	end component;

--	component extbus_gpio
--	port (
--		ext_bus		: inout std_logic_vector(41 downto 0);
--		data_in		: in std_logic_vector(31 downto 0);
--		data_out	: out std_logic_vector(41 downto 0);
--		dir_out		: out std_logic_vector(41 downto 0);
--		data_low_sel	: in std_logic;
--		data_hi_sel	: in std_logic;
--		dir_low_sel	: in std_logic;
--		dir_hi_sel	: in std_logic;
--		wr		: in std_logic;
--		clk		: in std_logic;
--		rst		: in std_logic
--	);
--	end component;

	component mach3_main
	generic (
		fifo_depth	: natural := 20
	);
	port (
		pulser_bus	: out std_logic_vector(11 downto 0);
		sp_dir		: out std_logic;
		sp_step		: out std_logic;
		axes_active	: out std_logic_vector(5 downto 0);
		irq_home	: out std_logic_vector(5 downto 0);
		irq_probe	: out std_logic;
		irq_cycle	: out std_logic;
		irq_done	: out std_logic;
		irq_fault	: out std_logic;
		irq_fovf	: out std_logic;
		idx		: out index_regs;
		cfg_reg		: out std_logic_vector(31 downto 0);
		cfg2_reg	: out std_logic_vector(31 downto 0);
		ctl_reg		: out std_logic_vector(15 downto 0);
		stat_reg	: out std_logic_vector(31 downto 0);
		enc_reg		: out std_logic_vector(31 downto 0);
		spcnt_reg	: out std_logic_vector(27 downto 0);
		stspctl_reg	: out std_logic_vector(26 downto 0);
		sp_idx_pulse	: out std_logic;
		sp_tim_pulse	: out std_logic;
		fovf_clr	: in std_logic;
		enc_data	: in std_logic_vector(7 downto 0);
		data_in		: in std_logic_vector(31 downto 0);
		din_reg		: in std_logic_vector(31 downto 0);
		idx_sel		: in std_logic_vector(5 downto 0);
		cfg_sel		: in std_logic;
		cfg2_sel	: in std_logic;
		ctl_sel		: in std_logic;
		stat_sel	: in std_logic;
		fifo_sel	: in std_logic;
		enc_sel		: in std_logic;
		spctl_sel	: in std_logic;
		rd		: in std_logic;
		wr		: in std_logic;
		clk		: in std_logic;
		nrst		: in std_logic;
		m_rst		: out std_logic
	);
	end component;
end mach3_components;

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.mach3_data_types.all;

package mach3_bits is
	component mach3_fifo
	generic (
		fifo_depth	: natural := 20
	);
	port (
		fifo_in		: in std_logic_vector(11 downto 0);
		fifo_out	: out std_logic_vector(11 downto 0);
		fifo_push	: in std_logic;
		fifo_pop	: in std_logic;
		fifo_purge	: in std_logic;
		fifo_run	: in std_logic;
		fifo_full	: out std_logic;
		fifo_empty	: out std_logic;
		fifo_ovf_irq	: out std_logic;
		fifo_ovf_clr	: in std_logic;
		fifo_room	: out std_logic_vector(4 downto 0);
		clk		: in std_logic;
		rst		: in std_logic
	);
	end component;

	component mach3_index
	port (
		data_in		: in std_logic_vector(31 downto 0);
		steps		: in std_logic_vector(5 downto 0);
		step_stb	: in std_logic;
		dirs		: in std_logic_vector(5 downto 0);
		idx_regs	: out index_regs;
		home_sw_hit	: in std_logic_vector(5 downto 0);
		probe_hit	: in std_logic;
		idx_wr		: in std_logic_vector(5 downto 0);
		clk		: in std_logic;
		rst		: in std_logic
	);
	end component;

	component mach3_switches
	port (
		sw_probe	: in std_logic;
		sw_home		: in std_logic_vector(5 downto 0);
		sw_lim_neg	: in std_logic_vector(5 downto 0);
		sw_lim_pos	: in std_logic_vector(5 downto 0);
		lim_neg_hit	: out std_logic_vector(5 downto 0);
		lim_pos_hit	: out std_logic_vector(5 downto 0);
		probe_hit	: out std_logic;
		home_hit	: out std_logic_vector(5 downto 0);
		lim_neg_clr	: in std_logic_vector(5 downto 0);
		lim_pos_clr	: in std_logic_vector(5 downto 0);
		probe_clr	: in std_logic;
		home_clr	: in std_logic_vector(5 downto 0);
		rst		: in std_logic;
		clk		: in std_logic
	);
	end component;

	component mach3_fsm is
	port (
		cycle_div	: in std_logic_vector(10 downto 0);
		fifo_pop	: out std_logic;
		fifo_empty	: in std_logic;
		tmr_req		: out std_logic;
		fsm_busy	: out std_logic;
		step_on		: out std_logic;
		step_stb	: out std_logic;
		fsm_run		: in std_logic;
		spindle_pulse	: in std_logic;
		wait4spindle	: in std_logic;
		ctl_wr		: in std_logic;
		clk		: in std_logic;
		rst		: in std_logic
	);
	end component;

	component mach3_encoders is
	port (
		enc_rd		: in std_logic;
		enc_data	: in std_logic_vector(7 downto 0);
		enc_counts	: out std_logic_vector(31 downto 0);
		clk		: in std_logic;
		rst		: in std_logic
	);
	end component;

	component mach3_spindle is
	port (
		sp_din_idx	: in std_logic;
		sp_din_tim	: in std_logic;
		sp_idx_ena	: in std_logic;
		sp_tim_ena	: in std_logic;
		idx_deadtime	: in std_logic_vector(9 downto 0);
		tim_deadtime	: in std_logic_vector(8 downto 0);
		stsp_ena	: in std_logic;
		sp_cnt		: out std_logic_vector(27 downto 0);
		stsp_ctl	: out std_logic_vector(26 downto 0);
		sp_idx_pulse	: out std_logic;
		sp_tim_pulse	: out std_logic;
		sp_stopped	: out std_logic;
		dir_out		: out std_logic;
		step_out	: out std_logic;
		stsp_wr		: in std_logic;
		data_in		: in std_logic_vector(31 downto 0);
		clk		: in std_logic;
		rst		: in std_logic
	);
	end component;
end mach3_bits;
